import os
import json
import hashlib
import getpass
import base64

if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from Orange.widgets.orangecontrib.AAIT.utils import MetManagement
    from Orange.widgets.orangecontrib.IO4IT.utils import secret_manager
else:
    from orangecontrib.AAIT.utils import MetManagement
    from orangecontrib.IO4IT.utils import secret_manager

def ask_secure():
    while True:
        resp = input("Secure? (Y/N) : ").strip().lower()
        if resp in ("y", "yes"):
            return True
        if resp in ("n", "no"):
            return False
        print("Please enter Y or N.")
#  FONCTIOSN D'OBFUSCATION GÉNÉRALES & CHEMIN
# Fonction pour générer une clé simple à partir du nom d'utilisateur
def get_user_key():
    try:
        username = getpass.getuser()
        #try:
        #username = os.getlogin()

        #except OSError:
        #username = getpass.getuser()

        if not username:
            raise ValueError("Nom d'utilisateur introuvable")

        # On dérive une clé simple (1 octet) depuis le hash du nom d'utilisateur
        digest = hashlib.sha256(username.encode("utf-8")).digest()
        key = digest[0]  # 1 octet pour XOR
        return key

    except Exception as e:
        raise RuntimeError(f"Erreur de génération de clé : {e}")


# Fonction simple de chiffrement/déchiffrement par XOR (non sécurisé mais obscurcissant)
def xor_crypt(data: str, key: int) -> str:
    return ''.join(chr(ord(c) ^ (key & 0xFF)) for c in data)


# Renvoie le chemin absolu vers le sous-dossier « aait_store/keys »
def get_keys_dir(type_key: str = "MICROSOFT_EXCHANGE_OAUTH2") -> str:
    """
    Retourne …/aait_store/keys/<type_key> sans // ni slash final,
    créé au besoin.  type_key ∈ {"IMAP4_SSL", "API", "NXP"}.
    """
    try:
        # normalise le chemin racine
        base = os.path.normpath(MetManagement.get_secret_content_dir())
        if os.path.basename(base) != "keys":
            base = os.path.join(base, "keys")
        dossier = os.path.normpath(os.path.join(base, type_key))
        os.makedirs(dossier, exist_ok=True)
        return dossier
    except Exception as e:
        raise RuntimeError(f"Erreur création/récupération dossier : {e}")





# GSTION IMAP
#je n'ai pas compris à quoi ça sert et ce n'est pas utilisé: à supprimer
""" 
def save_config(str_type,list_str=[]):
    # str_type==IMAP4_SSL -> list_str=[name,server_imap,mail]
    if str_type=="IMAP4_SSL":
        write_imap_config(list_str)
    return
"""


## va lire le fichier de white_list et black_list
## ce fichier et un json du type {"white_list":[], "black_list":[]}
## si pas de fichier ou juste white_liste ou black list il retourne uniquement celui présent
def lire_list_email(chemin_fichier):
    try:
        chemin_fichier = MetManagement.get_secret_content_dir() + chemin_fichier
        if not os.path.exists(chemin_fichier):
            return [[], []]
        # Lecture du fichier JSON
        with open(chemin_fichier, "r", encoding="utf-8") as f:
            contenu = json.load(f)
        return [
            contenu.get("white_list", []),
            contenu.get("black_list", [])
        ]
    except Exception as e:
        print(f"❌ Erreur lors de la lecture : {e}")
        return None


def enregistrer_config_imap4_ssl(agent, my_domain, password, interval_second, alias=""):
    try:
        dossier = get_keys_dir("IMAP4_SSL")
        # Crée le dossier s'il n'existe pas
        if not os.path.exists(dossier):
            os.makedirs(dossier)

        # Récupère l'adresse MAC et chiffre le mot de passe
        key = get_user_key()
        mdp_chiffre = xor_crypt(password, key)

        # Nom du fichier (remplace @ par _at_ pour éviter les problèmes)
        nom_fichier = os.path.join(dossier, f"{agent}{my_domain.replace('@', '_at_')}.json")
        if alias == "''" or alias == "\"\"":
            alias = ""

        # Contenu à écrire dans le fichier
        contenu = {
            "agent": agent,
            "domain": my_domain,
            "interval_second": interval_second,
            "password_encrypted": mdp_chiffre,
            "alias": alias
        }

        # Écriture du fichier
        with open(nom_fichier, "w", encoding="utf-8") as f:
            json.dump(contenu, f, indent=4)

        print(f"✅ Fichier enregistré : {nom_fichier}")
        return 0

    except Exception as e:
        print(f"❌ Erreur lors de l'enregistrement : {e}")
        return 1


def enregistrer_config_owa(mail, alias, server, username, password, interval):
    try:
        dossier = get_keys_dir("MICROSOFT_EXCHANGE_OWA")
        # Crée le dossier s'il n'existe pas
        if not os.path.exists(dossier):
            os.makedirs(dossier)

        # Récupère l'adresse MAC et chiffre le mot de passe
        key = get_user_key()
        mdp_chiffre = xor_crypt(password, key)

        # Nom du fichier (remplace @ par _at_ pour éviter les problèmes)
        nom_fichier = os.path.join(dossier, f"{alias.replace('@', '_at_')}.json")

        # Contenu à écrire dans le fichier
        contenu = {
            "mail": mail,
            "alias": alias,
            "server": server,
            "username": username,
            "password_encrypted": mdp_chiffre,
            "interval_second": interval
        }

        # Écriture du fichier
        with open(nom_fichier, "w", encoding="utf-8") as f:
            json.dump(contenu, f, indent=4)

        print(f"✅ Fichier enregistré : {nom_fichier}")
        return 0

    except Exception as e:
        print(f"❌ Erreur lors de l'enregistrement : {e}")
        return 1

def enregistrer_config_owa_secure(mail, alias, server, username, password, interval):
    try:
        dossier = get_keys_dir("MICROSOFT_EXCHANGE_OWA")
        # Crée le dossier s'il n'existe pas
        if not os.path.exists(dossier):
            os.makedirs(dossier)

        # Récupère l'adresse MAC et chiffre le mot de passe
        key = get_user_key()
        mdp_chiffre = xor_crypt(password, key)

        # Nom du fichier (remplace @ par _at_ pour éviter les problèmes)
        nom_fichier = os.path.join(dossier, f"{alias.replace('@', '_at_')}.sec")
        service_name=f"MICROSOFT_EXCHANGE_OWA__{alias.replace('@', '_at_')}"


        # Contenu à écrire dans le fichier
        contenu = {
            "mail": mail,
            "alias": alias,
            "server": server,
            "username": username,
            "password_encrypted": password,
            "interval_second": interval
        }
        sm = secret_manager.SecretManager(service_name)
        sm.store(contenu)
        # Écriture du fichier
        with open(nom_fichier, "w", encoding="utf-8") as f:
            pass

        print(f"✅ Fichier enregistré : {nom_fichier}")
        return 0

    except Exception as e:
        print(f"❌ Erreur lors de l'enregistrement : {e}")
        return 1



# Fonction pour lire le fichier de configuration et déchiffrer le mot de passe
def lire_config_imap4_ssl(chemin_fichier):
    # renvoie une liste =["agent","domain",mdp,"interval_second"]
    try:
        chemin_fichier = os.path.join(get_keys_dir("IMAP4_SSL"), chemin_fichier)
        # Lecture du fichier JSON
        with open(chemin_fichier, "r", encoding="utf-8") as f:
            contenu = json.load(f)

        # Récupère l'adresse MAC
        key = get_user_key()

        # Déchiffre le mot de passe
        mdp_dechiffre = xor_crypt(contenu["password_encrypted"], key)
        return [
            contenu["agent"],
            contenu["domain"],
            mdp_dechiffre,
            int(contenu["interval_second"]),
            contenu.get("alias", "")
        ]
    except Exception as e:
        print(f"❌ Erreur lors de la lecture : {e}")
        return None

def lire_config_oauth2(chemin_fichier):
    try:
        chemin_fichiers = os.path.join(get_keys_dir("MICROSOFT_EXCHANGE_OAUTH2"), chemin_fichier)
        with open(chemin_fichiers, "r", encoding="utf-8") as f:
            contenu = json.load(f)
        if "key" in contenu:
            key = contenu["key"]
            s_m = secret_manager.SecretManager(key)
            contenu_enregistre = s_m.load_all()
            return [contenu_enregistre["client_id"], contenu_enregistre["client_secret"], contenu_enregistre["tenant_id"], contenu_enregistre["user_email"]]
        else :
            key = get_user_key()
            client_id = xor_crypt(contenu["client_id_enc"], key)
            client_secret = xor_crypt(contenu["client_secret_enc"], key)
            tenant_id = xor_crypt(contenu["tenant_id_enc"], key)
            # Retourne les infos de configuration
            return [
                client_id,
                client_secret,
                tenant_id,
                contenu["user_email"]
            ]
    except Exception as e:
        print(f"❌ Erreur lors de la lecture : {e}")
        return None


def lire_config_owa(chemin_fichier):
    if len(chemin_fichier)<5:
        print(f"❌ Error path not correct", chemin_fichier)
        return None
    if chemin_fichier.endswith(".sec"):
        try:
            service="MICROSOFT_EXCHANGE_OWA__"+chemin_fichier[:-4]
            sm = secret_manager.SecretManager(service)
            contenu=sm.load_all()
            return [
                contenu["mail"],
                contenu["alias"],
                contenu["server"],
                contenu["username"],
                contenu["password_encrypted"],
                int(contenu["interval_second"])
            ]

        except Exception as e:
            print(f"❌ Erreur lors de la lecture : {e}")
            return None

    try:
        chemin_fichier = os.path.join(get_keys_dir("MICROSOFT_EXCHANGE_OWA"), chemin_fichier)
        print("chemin fichier", chemin_fichier)
        # Lecture du fichier JSON
        with open(chemin_fichier, "r", encoding="utf-8") as f:
            contenu = json.load(f)

        # Récupère l'adresse MAC
        key = get_user_key()

        # Déchiffre le mot de passe
        mdp_dechiffre = xor_crypt(contenu["password_encrypted"], key)
        return [
            contenu["mail"],
            contenu["alias"],
            contenu["server"],
            contenu["username"],
            mdp_dechiffre,
            int(contenu["interval_second"])
        ]

    except Exception as e:
        print(f"❌ Erreur lors de la lecture : {e}")
        return None


def enregistrer_config_cli_imap4_ssl():
    print("\n📝 Écriture d’un fichier de configuration :")
    agent = input("🤖 Nom de l’agent : ").strip()
    domaine = input("📨 @domain.com? : ").strip()
    mdp = input("📨mot de passe? : ").strip()
    interval = int(input("⏱️ Intervalle en secondes : ").strip())
    alias = input("Nom de l'alias : ").strip()
    if 0 != enregistrer_config_imap4_ssl(agent, domaine, mdp, interval, alias):
        print("erreur!")


def enregistrer_config_cli_owa():
    print("\n📝 Écriture d’un fichier de configuration owa :")
    mail = input("🤖 mail (nom@domain.com) : ").strip()
    alias = input("📨 alias (=mail apparant :(nom2@domain2.com) ").strip()
    server = input("server ? toto.titi.tata: ").strip()
    username = input("usernamme (domaine\\username): ").strip()
    mdp = input("password?: ").strip()
    interval = int(input("⏱️ Intervalle en secondes : ").strip())
    store = ask_secure()

    if alias == "''" or alias == "\"\"" or alias == "":
        alias = mail
    if not store:
        print("not secure")
        if 0 != enregistrer_config_owa(mail, alias, server, username, mdp, interval):
            print("erreur!")
        return
    if 0 != enregistrer_config_owa_secure(mail, alias, server, username, mdp, interval):
        print("erreur!")


def enregistrer_config_cli_oauth2():
    print("\n📝 Écriture d’un fichier de configuration OAuth2 :")
    key = get_user_key()
    client_id = input("🆔 Client ID : ").strip()
    client_secret = input("🔑 Client Secret : ").strip()
    tenant_id = input("🏢 Tenant ID (GUID Azure) : ").strip()
    user_email = input("📨 Adresse email de l'utilisateur Exchange : ").strip()
    client_id_enc = xor_crypt(client_id, key)
    client_secret_enc = xor_crypt(client_secret, key)
    tenant_id_enc = xor_crypt(tenant_id, key)
    nom_fichier = input("💾 Nom du fichier à enregistrer (ex: config_oauth2.json) : ").strip()
    choix_enregistrement = input("Voulez vous enregistrer en offuscation basique (1) ou enregistrer une clée dans "
                                 "le trousseau (2) ?")
    match choix_enregistrement:
        case "1":
            config = {
                "client_id_enc": client_id_enc,
                "client_secret_enc": client_secret_enc,
                "tenant_id_enc": tenant_id_enc,
                "user_email": user_email
            }
        case "2":
            cle = f"MICROSOFT_EXCHANGE_OAUTH2 {nom_fichier}"
            config = {"key": cle}
            config_to_save_in_key = {
                "client_id": client_id,
                "client_secret": client_secret,
                "tenant_id": tenant_id,
                "user_email": user_email
            }
            s_m = secret_manager.SecretManager(cle)
            s_m.store(config_to_save_in_key)




    try:
        dossier = get_keys_dir("MICROSOFT_EXCHANGE_OAUTH2")
        os.makedirs(dossier, exist_ok=True)
        chemin_fichier = os.path.join(dossier, nom_fichier)

        with open(chemin_fichier, "w", encoding="utf-8") as f:
            json.dump(config, f, indent=2)

        print(f"✅ Configuration enregistrée dans : {chemin_fichier}")
    except Exception as e:
        print(f"❌ Erreur lors de l’enregistrement : {e}")


def lire_config_cli_oauth2():
    chemin_fichier = input("📄 non fichier json (pas le chemin!) JSON : ").strip()
    config = lire_config_oauth2(chemin_fichier)
    if config == None:
        print("erreur")
    print(config)



def lecture_config_cli_owa():
    chemin_fichier = input("📄 non fichier json (pas le chemin!) JSON ou sec : ").strip()
    config = lire_config_owa(chemin_fichier)

    if config == None:
        print("erreur")
    print(config)


def lire_config_cli_imap4_ssl():
    chemin_fichier = input("📄 non fichier json (pas le chemin!) JSON : ").strip()
    config = lire_config_imap4_ssl(chemin_fichier)

    if config == None:
        print("erreur")
    print(config)


# Gestion clés API          (HARD dossier aait_store/keys)
# Enregistre un fichier JSON {service, api_key_encrypted, description}
def enregistrer_config_api(service_name, api_key, description=""):
    try:
        # Clé « personnelle » (1 octet) et chiffrement XOR
        key = get_user_key()
        api_key_enc = xor_crypt(api_key, key)

        contenu = {
            "service": service_name,
            "api_key_encrypted": api_key_enc,
            "description": description
        }

        chemin_fic = os.path.join(get_keys_dir("API"), f"{service_name}.json")
        with open(chemin_fic, "w", encoding="utf-8") as fp:
            json.dump(contenu, fp, indent=4)

        #print(f"✅ Fichier enregistré : {chemin_fic}")
        #print(get_user_key())
        return 0
    except Exception as e:
        print(f"❌ Erreur d’enregistrement : {e}")
        return 1


# Lecture + déchiffrement → dict {"service", "api_key", "description"}
def lire_config_api(service_name):
    try:
        chemin_fic = os.path.join(get_keys_dir("API"), f"{service_name}.json")
        with open(chemin_fic, "r", encoding="utf-8") as fp:
            contenu = json.load(fp)

        key = get_user_key()
        api_key_plain = xor_crypt(contenu["api_key_encrypted"], key)
        #print(api_key_plain)
        #print(get_user_key())

        return {
            "service": contenu["service"],
            "api_key": api_key_plain,
            "description": contenu.get("description", "")
        }
    except FileNotFoundError:
        print("❌ Fichier introuvable.")
        return None
    except Exception as e:
        print(f"❌ Erreur lors de la lecture : {e}")
        return None


def enregistrer_config_cli_api():
    print("\n📝 Écriture d’une clé API :")
    service = input("🔖 Nom du service : ").strip()
    api_key = input("🔑 Clé API         : ").strip()
    desc = input("✏️  Description      : ").strip()
    if 0 != enregistrer_config_api(service, api_key, desc):
        print("erreur!")


def lire_config_cli_api(service=""):
    if service == "":
        service = input("🔖 Nom du service : ").strip()
    try:
        cfg = lire_config_api(service)
        if cfg is None:
            print("erreur")
            return
        print(f"\n📄 service     : {cfg['service']}")
        print(f"🔑 clé API     : {cfg['api_key']}")
        if cfg['description']:
            print(f"📝 description : {cfg['description']}")
        return cfg['api_key']
    except Exception as e:
        print(f"❌ Erreur lors de la lecture : {e}")
        return None


# Gestion d’éléments de nxp (DOSSIER_NODE_ID, SERVEUR, USERNAME, PASSWORD)  (HARD dossier aait_store/keys)

def enregistrer_config_nxp(
        dossier_node_id: str,
        serveur: str,
        username: str,
        password: str,
        description: str = ""
) -> int:
    try:
        key = get_user_key()
        password_enc = xor_crypt(password, key)

        contenu = {
            "dossier_node_id": dossier_node_id,
            "serveur": serveur,
            "username": username,
            "password_encrypted": password_enc,
            "description": description
        }

        # ⬅️  Ici : plus de "conn_", juste {serveur}.json
        chemin = os.path.join(get_keys_dir("NXP"), f"{serveur}.json")
        with open(chemin, "w", encoding="utf-8") as f:
            json.dump(contenu, f, indent=4)

        print(f"✅ Fichier enregistré : {chemin}")
        return 0
    except Exception as e:
        print(f"❌ Erreur d’enregistrement : {e}")
        return 1


def lire_config_nxp(serveur: str) -> dict | None:
    try:
        chemin = os.path.join(get_keys_dir("NXP"), f"{serveur}.json")  # ⬅️ même logique
        with open(chemin, "r", encoding="utf-8") as f:
            contenu = json.load(f)

        key = get_user_key()
        password_plain = xor_crypt(contenu["password_encrypted"], key)

        return {
            "dossier_node_id": contenu["dossier_node_id"],
            "serveur": contenu["serveur"],
            "username": contenu["username"],
            "password": password_plain,
            "description": contenu.get("description", "")
        }
    except FileNotFoundError:
        print("❌ Fichier introuvable.")
        return None
    except Exception as e:
        print(f"❌ Erreur de lecture : {e}")
        return None


def enregistrer_config_cli_nxp():
    print("\n📝 Écriture d’une connexion nxp :")
    dossier_node_id = input("📦 DOSSIER_NODE_ID : ").strip()
    serveur = input("🌐 SERVEUR         : ").strip()
    username = input("👤 USERNAME        : ").strip()
    password = getpass.getpass("🔑 PASSWORD        : ").strip()
    description = input("✏️  Description     : ").strip()
    enregistrer_config_nxp(
        dossier_node_id,
        serveur,
        username,
        password,
        description
    )


def lire_config_cli_nxp():
    serveur = input("🌐 SERVEUR : ").strip()  # ⬅️ on demande le serveur
    cfg = lire_config_nxp(serveur)
    if cfg is None:
        print("erreur")
        return
    print(f"\n📄 dossier_node_id : {cfg['dossier_node_id']}")
    print(f"🌐 serveur         : {cfg['serveur']}")
    print(f"👤 username        : {cfg['username']}")
    print(f"🔑 password        : {cfg['password']}")
    if cfg["description"]:
        print(f"📝 description     : {cfg['description']}")


if __name__ == "__main__":
    print("add --no-log-tigger")
    print("1) ecrire fichier IMAP4_SSL")
    print("2) dechiffer fichier IMAP4_SSL")
    print("3) Écrire fichier CLÉ API")
    print("4) Déchiffrer fichier CLÉ API")
    print("5) Écrire fichier NXP")
    print("6) Déchiffrer fichier NXP")
    print("7) Écrire fichier Microsoft Exchange (OWA)")
    print("8) Déchiffrer fichier Microsoft Exchange (OWA)")
    print("9) Écrire fichier Microsoft Exchange (OAuth2)")
    print("10) Déchiffrer fichier Microsoft Exchange (OAuth2)")



    choix = input("👉 Que faire ? [1-10] : ").strip()

    if choix == "1":
        enregistrer_config_cli_imap4_ssl()
    elif choix == "2":
        lire_config_cli_imap4_ssl()
    elif choix == "3":
        enregistrer_config_cli_api()
    elif choix == "4":
        lire_config_cli_api()
    elif choix == "5":
        enregistrer_config_cli_nxp()
    elif choix == "6":
        lire_config_cli_nxp()
    elif choix == "7":
        enregistrer_config_cli_owa()
    elif choix == "8":
        lecture_config_cli_owa()
    elif choix == "9":
        enregistrer_config_cli_oauth2()
    elif choix == "10":
        lire_config_cli_oauth2()



    else:
        print("❌ Choix invalide. Réessayez.\n")



import os
import sys
import time
from pathlib import Path

from AnyQt.QtWidgets import QApplication
from Orange.widgets import widget
from Orange.widgets.utils.signals import Input, Output
from Orange.data import Domain, StringVariable, Table

# --- Docling Core and Docling ---
from docling.document_converter import DocumentConverter, AudioFormatOption
from docling.datamodel.base_models import InputFormat, ConversionStatus
from docling.pipeline.asr_pipeline import AsrPipeline
from docling.datamodel.pipeline_options import AsrPipelineOptions
from docling.datamodel import asr_model_specs
from docling.datamodel.document import ConversionResult

# --- Orange contrib Imports ---
if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from Orange.widgets.orangecontrib.AAIT.utils.thread_management import Thread
    from Orange.widgets.orangecontrib.AAIT.utils.import_uic import uic
else:
    from orangecontrib.AAIT.utils.thread_management import Thread
    from orangecontrib.AAIT.utils.import_uic import uic


# ----------------- Worker Function (Conversion ASR) ------------------
def _asr_conversion(file_path_str: str) -> list:
    """ASR conversion for a single MP3 file."""
    t0 = time.time()
    src = Path(file_path_str)

    # Define output directory and file
    out_dir = src.parent / "asr_md"
    out_dir.mkdir(parents=True, exist_ok=True)
    out_md = out_dir.joinpath(f"{src.stem}.md")

    # Don't re-process if output file exists
    if out_md.exists():
        status = "ok"
        message = "cached"
        duration = time.time() - t0
        return [str(src), str(out_md), status, f"{duration:.2f}", message]

    status, message = "ok", ""  # Default status

    try:
        # Docling ASR configuration
        pipeline_options = AsrPipelineOptions()
        pipeline_options.asr_options = asr_model_specs.WHISPER_BASE

        converter = DocumentConverter(
            format_options={
                InputFormat.AUDIO: AudioFormatOption(
                    pipeline_cls=AsrPipeline,
                    pipeline_options=pipeline_options,
                )
            }
        )

        # Perform conversion
        result: ConversionResult = converter.convert(src)

        if result.status == ConversionStatus.SUCCESS:
            # New check: ensure text exists before exporting
            if hasattr(result.document, 'texts') and result.document.texts:
                markdown_content = result.document.export_to_markdown()
                out_md.write_text(markdown_content, encoding="utf-8")
                message = ""  # Conversion successful with text
            else:
                # Case where conversion is "successful" but no text is found
                markdown_content = "[Warning] Conversion réussie, mais aucun texte n'a été transcrit (audio silencieux ?)"
                out_md.write_text(markdown_content, encoding="utf-8")
                status = "ok"  # Still technically ok, but with a warning
                message = "Conversion réussie, mais aucun texte n'a été transcrit (audio silencieux ?)"
        else:
            status = "nok"
            message = f"Conversion failed with status: {result.status}"
            try:
                out_md.write_text(f"[Erreur ASR] {message}", encoding="utf-8")
            except Exception:
                pass

    except Exception as e:
        status = "nok"
        message = f"Error: {e}"
        try:
            out_md.write_text(f"[Erreur ASR] {message}", encoding="utf-8")
        except Exception:
            pass

    duration = time.time() - t0
    return [str(src), str(out_md), status, f"{duration:.2f}", message]


class OWDoclingASR(widget.OWWidget):
    name = "Docling ASR"
    description = "Convertit des fichiers audio (MP3) en texte (Markdown) via Docling."
    category = "IO4IT"  # You may need to change the category
    icon = "icons/asr.png"  # Assurez-vous que l'icône existe dans un dossier 'icons'
    gui = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/owdoclingasr.ui")
    want_control_area = True
    priority = 1005

    class Inputs:
        data = Input("Files Table", Table)

    class Outputs:
        data = Output("ASR Markdown", Table)

    def __init__(self):
        super().__init__()
        self.setFixedWidth(470)
        self.setFixedHeight(300)
        uic.loadUi(self.gui, self)

        self.data = None
        self.thread = None
        self.result = None
        self.post_initialized()

    @Inputs.data
    def set_data(self, in_data: Table | None):
        self.data = in_data
        self.run()

    def run(self):
        if self.thread is not None and self.thread.isRunning():
            self.thread.safe_quit()

        if self.data is None:
            self.Outputs.data.send(None)
            return

        self.error("")
        try:
            self.data.domain["file_path"]
        except KeyError:
            self.error("La colonne 'file_path' est requise dans les données d'entrée.")
            return

        self.progressBarInit()

        # Récupérer et filtrer les chemins de fichiers
        try:
            paths = [Path(str(x)) for x in self.data.get_column("file_path")]
        except Exception as e:
            self.error(f"Impossible de lire la colonne 'file_path': {e}")
            self.Outputs.data.send(None)
            return

        files = [p for p in paths if p.exists() and p.suffix.lower() == ".mp3"]
        if not files:
            self.error("Aucun fichier .mp3 valide n'a été trouvé.")
            self.Outputs.data.send(None)
            self.progressBarFinished()
            return

        # Démarrer le thread de conversion
        self.thread = Thread(self._run_conversion, files)
        self.thread.progress.connect(self.handle_progress)
        self.thread.result.connect(self.handle_result)
        self.thread.finish.connect(self.handle_finish)
        self.thread.start()

    def _run_conversion(self, files: list[Path], progress_callback):
        """Fonction principale pour exécuter la conversion séquentielle."""
        results = []
        total_files = len(files)

        for i, file_path in enumerate(files):
            row = _asr_conversion(str(file_path))
            results.append(row)
            progress_callback((i + 1) / total_files * 100)

        # Construction de la table de sortie
        domain = Domain([], metas=[
            StringVariable("src_path"),
            StringVariable("output_md"),
            StringVariable("status"),
            StringVariable("duration_sec"),
            StringVariable("message"),
        ])

        table = Table.from_list(domain, results)
        return table

    def handle_progress(self, value: float) -> None:
        self.progressBarSet(value)

    def handle_result(self, result: Table):
        try:
            self.result = result
            self.Outputs.data.send(result)
        except Exception as e:
            print("An error occurred when sending out_data:", e)
            self.Outputs.data.send(None)
            return

    def handle_finish(self):
        self.progressBarFinished()

    def post_initialized(self):
        pass


if __name__ == "__main__":
    app = QApplication(sys.argv)
    my_widget = OWDoclingASR()
    my_widget.show()
    if hasattr(app, "exec"):
        app.exec()
    else:
        app.exec_()


import os
import sys
from pathlib import Path

from AnyQt.QtWidgets import QApplication
from AnyQt.QtCore import pyqtSignal
from Orange.data import Domain, StringVariable, Table, DiscreteVariable
from Orange.widgets import widget
from Orange.widgets.utils.signals import Input, Output

# --- Ajout pour l'écriture Excel ---
from openpyxl import Workbook
import pypdf

# Les imports sont adaptés pour correspondre au style de l'autre script
if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from Orange.widgets.orangecontrib.IO4IT.utils import utils_md
    from Orange.widgets.orangecontrib.AAIT.utils.thread_management import Thread
    from Orange.widgets.orangecontrib.AAIT.utils.import_uic import uic
else:
    from orangecontrib.IO4IT.utils import utils_md
    from orangecontrib.AAIT.utils.thread_management import Thread
    from orangecontrib.AAIT.utils.import_uic import uic


class OWPdfType(widget.OWWidget):
    name = "PDF Type"
    description = "Checks if a PDF is text-based or image-based"
    category = "AAIT - TOOLBOX"
    icon = "icons/check_pdf.png"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/check_pdf.png"
    gui = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/owpdftype.ui")
    want_control_area = False
    priority = 1002

    MAX_PAGES_TO_CHECK = 5

    # New signal to send single status updates from the thread
    status_update_signal = pyqtSignal(list)

    class Inputs:
        data = Input("PDF Table", Table)

    class Outputs:
        text_data = Output("Text PDF Table", Table)
        image_data = Output("Image PDF Table", Table)
        status_data = Output("Status Table", Table)

    def __init__(self):
        super().__init__()
        self.setFixedWidth(470)
        self.setFixedHeight(300)
        uic.loadUi(self.gui, self)

        self.data = None
        self.thread = None
        self.autorun = True
        self.result = None
        self.processed_statuses = []
        self.post_initialized()

    @Inputs.data
    def set_data(self, in_data: Table | None):
        self.data = in_data
        if self.autorun:
            self.run()

    def run(self):
        if self.thread is not None and self.thread.isRunning():
            self.thread.safe_quit()

        if self.data is None:
            self.Outputs.text_data.send(None)
            self.Outputs.image_data.send(None)
            self.Outputs.status_data.send(None)
            QApplication.quit()
            return

        self.error("")
        try:
            self.data.domain["file_path"]
        except KeyError:
            self.error("You need a 'file_path' column in input data.")
            QApplication.quit()
            return

        if type(self.data.domain["file_path"]).__name__ != 'StringVariable':
            self.error("'file_path' column needs to be a Text.")
            QApplication.quit()
            return

        self.progressBarInit()
        self.processed_statuses = []
        self.status_update_signal.connect(self.handle_status_update)
        self.thread = Thread(self._process_pdfs, self.data, status_callback=self.status_update_signal.emit)
        self.thread.progress.connect(self.handle_progress)
        self.thread.result.connect(self.handle_result)
        self.thread.finish.connect(self.handle_finish)
        self.thread.start()

    def _process_pdfs(self, in_data: Table, progress_callback: callable, status_callback: callable) -> tuple[
        Table | None, Table | None]:

        paths = [str(x) for x in in_data.get_column("file_path")]

        excel_output_dir = Path.cwd() / "pdf_check_results"
        if paths:
            first_file_path = Path(paths[0])
            excel_output_dir = first_file_path.parent / "pdf_check_results"

        excel_output_dir.mkdir(parents=True, exist_ok=True)

        base_name = "pdf_check_results"
        excel_path = excel_output_dir / f"{base_name}.xlsx"
        counter = 1
        while excel_path.exists():
            excel_path = excel_output_dir / f"{base_name}_{counter}.xlsx"
            counter += 1

        wb = Workbook()
        ws = wb.active
        ws.title = "PDF Check Results"
        headers = ["file_path", "status", "details"]
        ws.append(headers)

        text_indices = []
        image_indices = []

        total_files = len(paths)
        for i, p in enumerate(paths):
            progress_callback(i / total_files * 100)
            fp = Path(p)
            result_row = [p, "", ""]

            if not fp.exists() or fp.suffix.lower() != ".pdf":
                result_row[1] = "ko"
                result_row[2] = "Invalid file or not a PDF"
                status_callback(result_row)
                ws.append(result_row)
                wb.save(excel_path)
                continue

            try:
                # Vérification de l'attribut is_encrypted de PyPDF avant de traiter le fichier.
                reader = pypdf.PdfReader(fp)
                if reader.is_encrypted:
                    result_row[1] = "ko"
                    result_row[2] = "Error: PDF is password protected."
                else:
                    is_text = utils_md.is_pdf_text_based(fp)
                    if is_text:
                        text_indices.append(i)
                        result_row[1] = "ok"
                        result_row[2] = "Text-based PDF"
                    else:
                        image_indices.append(i)
                        result_row[1] = "ok"
                        result_row[2] = "Image-based PDF"
            except Exception as e:
                result_row[1] = "ko"
                result_row[2] = f"Error: {str(e)}"
            finally:
                status_callback(result_row)
                ws.append(result_row)
                wb.save(excel_path)

        progress_callback(100)

        if not text_indices:
            text_table = None
        else:
            text_table = in_data[text_indices]

        if not image_indices:
            image_table = None
        else:
            image_table = in_data[image_indices]

        return text_table, image_table

    def handle_progress(self, value: float) -> None:
        self.progressBarSet(value)

    def handle_status_update(self, new_status: list):
        self.processed_statuses.append(new_status)
        status_domain = Domain(
            [],
            metas=[
                StringVariable("file_path"),
                DiscreteVariable("status", values=["ok", "ko"]),
                StringVariable("details")
            ]
        )
        status_table = Table.from_list(status_domain, self.processed_statuses)
        self.Outputs.status_data.send(status_table)

    def handle_result(self, result):
        try:
            text_table, image_table = result
            self.Outputs.text_data.send(text_table)
            self.Outputs.image_data.send(image_table)
        except Exception as e:
            print("An error occurred when sending out_data:", e)
            self.Outputs.text_data.send(None)
            self.Outputs.image_data.send(None)

    def handle_finish(self):
        print("PDF Type check finished")
        self.progressBarFinished()

    def post_initialized(self):
        pass


if __name__ == "__main__":
    app = QApplication(sys.argv)
    my_widget = OWPdfType()
    my_widget.show()
    if hasattr(app, "exec"):
        app.exec()
    else:
        app.exec_()


from datetime import datetime
from typing import List, Dict
from ddgs import DDGS
import os
import sys
import Orange
import re
from Orange.widgets.widget import Input, Output
from AnyQt.QtWidgets import QApplication, QPushButton, QLineEdit, QSpinBox, QDoubleSpinBox
import json
from Orange.widgets.settings import Setting

if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from Orange.widgets.orangecontrib.AAIT.utils import thread_management, base_widget
    from Orange.widgets.orangecontrib.HLIT_dev.remote_server_smb import convert
else:
    from orangecontrib.AAIT.utils import thread_management, base_widget
    from orangecontrib.HLIT_dev.remote_server_smb import convert

class WebSearch(base_widget.BaseListWidget):
    name = "WebSearch"
    description = "Search url website from a query with DDG."
    icon = "icons/websearch.png"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/websearch.png"
    priority = 3000
    gui = ""
    want_control_area = False
    category = "AAIT - TOOLBOX"
    gui = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/owwebsearch.ui")
    # Settings
    selected_column_name = Setting("content")
    region = Setting('fr-fr')
    time_range = Setting('y')
    max_results = Setting(20)
    relevance_threshold = Setting(0.3)

    class Inputs:
        data = Input("Data", Orange.data.Table)

    @Inputs.data
    def set_data(self, in_data):
        self.data = in_data
        if in_data is None:
            self.Outputs.data.send(None)
            return
        if self.data:
            self.var_selector.add_variables(self.data.domain)
            self.var_selector.select_variable_by_name(self.selected_column_name)
        self.run()

    class Outputs:
        data = Output("Data", Orange.data.Table)


    def __init__(self):
        super().__init__()
        # Qt Management
        self.setFixedWidth(500)
        self.setFixedHeight(600)

        self.edit_region = self.findChild(QLineEdit, 'boxRegion')
        self.edit_region.setPlaceholderText("Region")
        self.edit_region.setText(self.region)
        self.edit_region.editingFinished.connect(self.update_parameters)

        self.edit_time_range = self.findChild(QLineEdit, 'boxTimeRange')
        self.edit_time_range.setPlaceholderText("Time Range")
        self.edit_time_range.setText(self.time_range)
        self.edit_time_range.editingFinished.connect(self.update_parameters)

        self.edit_max_results = self.bind_spinbox("boxMaxResults", self.max_results)
        self.edit_relevance_threshold = self.bind_spinbox("boxRelevanceThreshold", self.relevance_threshold, is_double=True)

        self.pushButton_run =self.findChild(QPushButton, 'pushButton_send')
        self.pushButton_run.clicked.connect(self.run)
        self.load_config()

    def bind_spinbox(self, name, value, is_double=False):
        widget_type = QDoubleSpinBox if is_double else QSpinBox
        box = self.findChild(widget_type, name)
        box.setValue(value)
        box.editingFinished.connect(self.update_parameters)
        return box

    def update_parameters(self):
        self.max_results = self.edit_max_results.value()
        self.relevance_threshold = self.edit_relevance_threshold.value()
        self.time_range = self.edit_time_range.text()
        self.region = self.edit_region.text()

    def load_config(self):
        config_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "../utils/config.json")
        with open(config_path, "r", encoding="utf-8") as f:
            config = json.load(f)
        self.domain_context = config["domain_context"]
        self.stop_words = set(config["stop_words"])

    def detect_domain(self, query: str):
        """Détecte les domaines dans la requête"""
        query_lower = query.lower()
        detected = []

        for domain_key in self.domain_context.keys():
            if domain_key in query_lower:
                detected.append(domain_key)

        return detected

    def get_contextual_terms(self, query: str):
        """Récupère les termes contextuels basés sur le domaine"""
        domains = self.detect_domain(query)

        if not domains:
            return []

        context_terms = []
        for domain in domains:
            terms = self.domain_context.get(domain, [])[:3]
            context_terms.extend(terms)

        return context_terms

    def optimize_query(self, query: str):
        """Génère des variations optimisées"""
        query = self.clean_query(query)
        variations = []

        # Détecter noms scientifiques et dates
        scientific_names = self.detect_scientific_name(query)
        temporal_exprs = self.detect_temporal_expressions(query)
        key_phrases = self.extract_key_phrases(query)

        words = query.split()
        important_words = [
            w for w in words
            if len(w) > 3 and w.lower() not in self.stop_words
        ]

        # Noms scientifiques entre guillemets
        if scientific_names:
            for sci_name in scientific_names:
                variations.append(f'"{sci_name}"')
                other_words = [w for w in important_words if w not in sci_name.split()]
                context = other_words[:3] + temporal_exprs
                if context:
                    variations.append(f'"{sci_name}" {" ".join(context)}')

        # Avec expressions temporelles
        if temporal_exprs and not scientific_names:
            if len(important_words) >= 1:
                non_temporal_words = []
                for word in important_words:
                    is_part_of_temporal = False
                    for temp_expr in temporal_exprs:
                        if word.lower() in temp_expr.lower():
                            is_part_of_temporal = True
                            break
                    if not is_part_of_temporal:
                        non_temporal_words.append(word)

                if non_temporal_words and temporal_exprs:
                    variations.append(f"{' '.join(non_temporal_words[:3])} {' '.join(temporal_exprs)}")

                if key_phrases and non_temporal_words:
                    main_phrase = key_phrases[0]
                    contains_temporal = any(temp in main_phrase for temp in temporal_exprs)
                    if not contains_temporal and len(main_phrase.split()) >= 2:
                        variations.append(f'"{main_phrase}" {" ".join(temporal_exprs)}')

        # Phrase clé entre guillemets
        if key_phrases and not scientific_names and not temporal_exprs:
            main_phrase = key_phrases[0]
            if len(main_phrase.split()) >= 2:
                variations.append(f'"{main_phrase}"')
                if len(key_phrases) > 1:
                    variations.append(f'"{main_phrase}" {key_phrases[1]}')

        # Ultra-simplifié
        if len(important_words) >= 2:
            simplified = ' '.join(important_words[:4])
            if temporal_exprs:
                for temp_expr in temporal_exprs:
                    if temp_expr.lower() not in simplified.lower():
                        simplified = f"{simplified} {temp_expr}"
            variations.append(simplified)
        elif len(important_words) == 1 and temporal_exprs:
            variations.append(f"{important_words[0]} {' '.join(temporal_exprs)}")

        # Enrichissement contextuel
        context_terms = self.get_contextual_terms(query)
        if context_terms and important_words:
            enriched_parts = important_words[:3] + context_terms[:2]
            if temporal_exprs:
                enriched_parts.extend(temporal_exprs)
            enriched = ' '.join(enriched_parts)
            variations.append(enriched)

        variations.append(query)

        # Dédupliquer
        seen = set()
        unique_variations = []
        for v in variations:
            v_clean = v.strip()
            if v_clean and v_clean not in seen and len(v_clean.split()) <= 10:
                seen.add(v_clean)
                unique_variations.append(v_clean)

        return unique_variations[:6]

    def calculate_relevance(self, query: str, title: str, snippet: str):
        """Calcule un score de pertinence"""
        query_lower = query.lower()
        title_lower = title.lower()
        snippet_lower = snippet.lower()

        query_words = [
            w for w in query_lower.split()
            if len(w) > 3 and w not in self.stop_words
        ]

        if not query_words:
            return 0.5

        score = 0.0
        max_score = len(query_words)

        for word in query_words:
            if word in title_lower:
                score += 0.6
            elif word in snippet_lower:
                score += 0.4
            else:
                word_norm = self.normalize_text(word)
                title_norm = self.normalize_text(title_lower)
                snippet_norm = self.normalize_text(snippet_lower)

                if word_norm in title_norm:
                    score += 0.5
                elif word_norm in snippet_norm:
                    score += 0.3

        return min(score / max_score, 1.0)

    def normalize_text(self, text: str):
        """Normalise le texte"""
        accent_map = {
            'é': 'e', 'è': 'e', 'ê': 'e', 'ë': 'e',
            'à': 'a', 'â': 'a', 'ä': 'a',
            'î': 'i', 'ï': 'i',
            'ô': 'o', 'ö': 'o',
            'ù': 'u', 'û': 'u', 'ü': 'u',
            'ç': 'c', 'ñ': 'n'
        }

        result = text.lower()
        for old, new in accent_map.items():
            result = result.replace(old, new)

        return result

    def filter_by_relevance(self, results: List[Dict], query: str):
        """Filtre avec vérification de fraîcheur"""
        from datetime import datetime, timedelta

        scored_results = []
        current_year = datetime.now().year

        for result in results:
            score = self.calculate_relevance(
                query,
                result.get('title', ''),
                result.get('body', result.get('snippet', ''))
            )

            # Pénaliser les vieux contenus
            title = result.get('title', '')
            snippet = result.get('body', result.get('snippet', ''))

            # Chercher des années dans le contenu
            years_found = re.findall(r'\b(20\d{2})\b', title + ' ' + snippet)
            if years_found:
                max_year = max(int(y) for y in years_found)
                year_diff = current_year - max_year

                # Pénalité selon l'ancienneté
                if year_diff > 2:
                    score *= 0.3
                elif year_diff > 1:
                    score *= 0.7

            result['relevance_score'] = score
            scored_results.append(result)

        filtered = [r for r in scored_results if r['relevance_score'] >= self.relevance_threshold]
        filtered.sort(key=lambda x: x['relevance_score'], reverse=True)

        return filtered

    def detect_scientific_name(self, query: str):
        """Détecte les noms scientifiques"""
        excluded_words = {'prix', 'cours', 'marché', 'janvier', 'février', 'mars', 'avril', 'mai', 'juin', 'juillet',
                          'août', 'septembre', 'octobre', 'novembre', 'décembre', 'année', 'mois', 'jour', 'monde',
                          'france', 'europe', 'production', 'commerce', 'export', 'import', 'aquaculture',
                          'distribution', 'habitat', 'recherche', 'étude', 'analyse', 'rapport'}
        context_words = {'prix', 'cours', 'marché', 'production', 'recherche', 'étude', 'analyse', 'rapport', 'habitat',
                         'aquaculture', 'distribution', 'ecology', 'biology', 'genetic', 'fishery', 'cultivation',
                         'harvest', 'spawning'}
        pattern = r'\b([a-zA-Z]{4,})\s+([a-zA-Z]{4,})\b'
        matches = re.finditer(pattern, query.lower())

        scientific_names = []
        for match in matches:
            word1, word2 = match.groups()

            if word1 in excluded_words or word2 in excluded_words:
                continue

            if word1 in context_words and word2 in context_words:
                continue

            if word1 in context_words:
                continue

            latin_suffixes = ('us', 'a', 'is', 'um', 'ae', 'i', 'ica', 'ensis', 'anus', 'ina', 'ella', 'ina')
            has_latin_ending = word2.endswith(latin_suffixes) or word1.endswith(latin_suffixes)

            if has_latin_ending:
                normalized = f"{word1.capitalize()} {word2.lower()}"
                if normalized not in scientific_names:
                    scientific_names.append(normalized)

        return scientific_names

    def detect_temporal_expressions(self, query: str):
        """Détecte les expressions temporelles"""
        temporal_patterns = [
            r'\b(janvier|février|mars|avril|mai|juin|juillet|août|septembre|octobre|novembre|décembre)\s+\d{4}\b',
            r'\b\d{1,2}[/-]\d{1,2}[/-]\d{4}\b',
            r'\b[QT][1-4]\s+\d{4}\b',
            r'\b(S[12]|premier|second|1er|2ème)\s+(semestre|trimestre)\s+\d{4}\b',
            r'\b(20\d{2})\b',
            r'\b(cette|l\'|cette)\s+(année|semaine|mois)\b',
            r'\b(dernier|dernière|prochain|prochaine)\s+(année|semaine|mois|trimestre)\b',
        ]

        found = []
        remaining_query = query

        for pattern in temporal_patterns:
            matches = re.finditer(pattern, remaining_query, re.IGNORECASE)
            for match in matches:
                expr = match.group()
                found.append(expr)
                remaining_query = remaining_query.replace(expr, ' ' * len(expr))

        return found

    def extract_key_phrases(self, query: str):
        """Extrait les phrases clés"""
        temporal_exprs = self.detect_temporal_expressions(query)

        temp_query = query
        temporal_tokens = {}
        for i, expr in enumerate(temporal_exprs):
            token = f"__TEMPORAL_{i}__"
            temporal_tokens[token] = expr
            temp_query = temp_query.replace(expr, token)

        words = temp_query.split()

        important_indices = []
        for i, word in enumerate(words):
            if word.startswith("__TEMPORAL_"):
                important_indices.append(i)
            elif len(word) > 3 and word.lower() not in self.stop_words:
                important_indices.append(i)

        phrases = []
        if not important_indices:
            return []

        current_phrase = [words[important_indices[0]]]
        last_idx = important_indices[0]

        for idx in important_indices[1:]:
            if idx - last_idx <= 2:
                for j in range(last_idx + 1, idx + 1):
                    if words[j].lower() not in self.stop_words or len(current_phrase) == 1 or words[j].startswith(
                            "__TEMPORAL_"):
                        current_phrase.append(words[j])
            else:
                if len(current_phrase) >= 1:
                    phrase = ' '.join(current_phrase)
                    for token, expr in temporal_tokens.items():
                        phrase = phrase.replace(token, expr)
                    phrases.append(phrase)
                current_phrase = [words[idx]]
            last_idx = idx

        if len(current_phrase) >= 1:
            phrase = ' '.join(current_phrase)
            for token, expr in temporal_tokens.items():
                phrase = phrase.replace(token, expr)
            phrases.append(phrase)

        return phrases

    def clean_query(self, query: str):
        """Nettoie la requête"""
        query = query.strip()

        generic_prefixes = [
            r'^(les?\s+)?informations?\s+(disponibles?\s+)?(sur|concernant|relatif|au sujet)\s+',
            r'^(je\s+)?(cherche|recherche|veux|voudrais|souhaite)\s+',
            r'^(peux-tu|pouvez-vous|trouve|trouver)\s+',
            r'^(donne-moi|donnez-moi)\s+',
        ]

        for pattern in generic_prefixes:
            query = re.sub(pattern, '', query, flags=re.IGNORECASE)

        return query.strip()

    def search(self, use_optimization: bool = True):
        all_results = []
        seen_urls = set()

        if use_optimization:
            query_variations = self.optimize_query(self.query)
            queries_to_try = query_variations
        else:
            queries_to_try = [self.query]

        for idx, q in enumerate(queries_to_try, 1):
            if len(all_results) >= self.max_results:
                break

            try:
                with DDGS() as ddgs:
                    search_results = list(ddgs.text(
                        q,
                        region=self.region,
                        safesearch='off',
                        timelimit=self.time_range,
                        max_results=min(50, self.max_results * 3)
                    ))
                    filtered = self.filter_by_relevance(search_results, self.query)

                    new_count = 0
                    for r in filtered:
                        if r['href'] not in seen_urls:
                            seen_urls.add(r['href'])

                            result = {
                                'url': r['href'],
                                'title': r['title'],
                                'snippet': r.get('body', ''),
                                'source': 'DuckDuckGo',
                                'query': self.query,
                                'query_variation': q,
                                'relevance_score': r['relevance_score'],
                                'fetched_at': datetime.now().isoformat(),
                                'rank': len(all_results) + 1
                            }

                            all_results.append(result)
                            new_count += 1

                            if len(all_results) >= self.max_results:
                                break

            except Exception as e:
                print(e)
                continue

        return all_results[:self.max_results]

    def run(self):
        self.error("")
        self.warning("")
        if self.data is None:
            self.Outputs.data.send(None)
            return

        if not self.selected_column_name in self.data.domain:
            self.warning(f'Previously selected column "{self.selected_column_name}" does not exist in your data.')
            return

        self.query = self.data.get_column(self.selected_column_name)[0]

        self.progressBarInit()
        self.thread = thread_management.Thread(self.search)
        self.thread.progress.connect(self.handle_progress)
        self.thread.result.connect(self.handle_result)
        self.thread.finish.connect(self.handle_finish)
        self.thread.start()

    def handle_progress(self, progress) -> None:
        value = progress[0]
        text = progress[1]
        if value is not None:
            self.progressBarSet(value)
        if text is None:
            self.textBrowser.setText("")
        else:
            self.textBrowser.insertPlainText(text)

    def handle_result(self, result):
        if result is None or len(result) == 0:
            self.Outputs.data.send(None)
            return
        data = convert.convert_json_implicite_to_data_table(result)
        self.Outputs.data.send(data)

    def handle_finish(self):
        self.progressBarFinished()

    def post_initialized(self):
        pass


if __name__ == "__main__":
    app = QApplication(sys.argv)
    my_widget = WebSearch()
    my_widget.show()

    if hasattr(app, "exec"):
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())


import os
import sys
import Orange
from Orange.widgets.widget import OWWidget, Input, Output
from AnyQt.QtWidgets import QApplication
import asyncio
import aiohttp
import html2text
from bs4 import BeautifulSoup

if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from Orange.widgets.orangecontrib.AAIT.utils import thread_management
    from Orange.widgets.orangecontrib.HLIT_dev.remote_server_smb import convert
    from Orange.widgets.orangecontrib.AAIT.utils.import_uic import uic
else:
    from orangecontrib.AAIT.utils.import_uic import uic
    from orangecontrib.HLIT_dev.remote_server_smb import convert
    from orangecontrib.AAIT.utils import thread_management

class ParseHMTL(OWWidget):
    name = "ParseHTML"
    description = "Parse website HTML. You need to provide url(s) in input."
    icon = "icons/html.png"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/html.png"
    priority = 3000
    gui = ""
    want_control_area = False
    category = "AAIT - TOOLBOX"
    gui = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/owparserhtml.ui")

    class Inputs:
        data = Input("Data", Orange.data.Table)

    @Inputs.data
    def set_data(self, in_data):
        if in_data is None:
            return
        if "url" not in in_data.domain:
            self.error("input table need a url column")
            return
        self.data = in_data
        self.url_data = in_data.get_column("url")
        self.run()

    class Outputs:
        data = Output("Data", Orange.data.Table)

    def __init__(self):
        super().__init__()
        # Qt Management
        self.setFixedWidth(500)
        self.setFixedHeight(400)
        uic.loadUi(self.gui, self)

        self.data = None
        self.thread = None
        self.markdown = True
        self.run()

    def update_parameters(self):
        return


    def parse_html(self):
        """Execute le parsing"""
        try:
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)
            results = loop.run_until_complete(self.parse_all_urls())
            loop.close()
            return results
        except Exception as e:
            self.error(str(e))
            return


    async def parse_all_urls(self, progress_callback=None):
        """Parse toutes les URLs de manière asynchrone"""
        results = []
        total = len(self.url_data)
        timeout = aiohttp.ClientTimeout(total=30)
        connector = aiohttp.TCPConnector(limit=5, limit_per_host=2)
        async with aiohttp.ClientSession(timeout=timeout, connector=connector,cookie_jar=aiohttp.CookieJar()) as session:
            for idx, url_data in enumerate(self.url_data):
                if progress_callback is not None:
                    progress_value = int((idx / total) * 100)
                    progress_callback(progress_value)
                try:
                    parsed = await self.parse_single_url(session, url_data)
                    results.append(parsed)

                except Exception as e:
                    results.append({
                        "url": url_data,
                        'content': '',
                        'meta_description': '',
                        'word_count': 0,
                        'status': f'error: {str(e)}'
                    })

        return results

    async def parse_single_url(self, session, url):
        """Parse une seule URL"""
        # Fetch HTML
        headers = {
                'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36',
                'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
                'Accept-Language': 'fr-FR,fr;q=0.9,en-US;q=0.8,en;q=0.7',
                'Accept-Encoding': 'gzip, deflate, br',
                'Connection': 'keep-alive',
                'Upgrade-Insecure-Requests': '1'
        }
        async with session.get(url, headers=headers) as response:
            if response.status != 200:
                raise Exception(f"HTTP {response.status}")

            html = await response.text()
        soup = BeautifulSoup(html, 'html.parser')

        meta_desc = ''
        meta_tag = soup.find('meta', attrs={'name': 'description'})
        if not meta_tag:
            meta_tag = soup.find('meta', property='og:description')
        if meta_tag:
            meta_desc = meta_tag.get('content', '')
        content = ''
        try:
            content = self._extract_main_content(soup)
        except Exception as e:
            print(e)
        word_count = len(content.split())
        return {
            "url": url,
            'content': content,
            'meta_description': meta_desc,
            'word_count': word_count,
            'status': 'success'
        }

    def _extract_main_content(self, soup):
        """Extrait le contenu principal et le convertit en Markdown"""
        main_selectors = [
            'article',
            'main',
            '[role="main"]',
            '.content',
            '.main-content',
            '#content',
            '.article-body',
            '.post-content'
        ]
        converter = html2text.HTML2Text()
        converter.ignore_links = False
        converter.ignore_images = False
        converter.body_width = 0
        for selector in main_selectors:
            main_elem = soup.select_one(selector)
            if main_elem:
                if self.markdown:
                    html = str(main_elem)
                    markdown = converter.handle(html)
                    if len(markdown.split()) > 100:
                        return markdown.strip()
                else:
                    paragraphs = main_elem.find_all('p')
                    if paragraphs:
                        text = ' '.join([p.get_text(strip=True) for p in paragraphs])
                        if len(text) > 100:
                                return text
        paragraphs = soup.find_all('p')
        if paragraphs:
            return ' '.join([p.get_text(strip=True) for p in paragraphs])

        return soup.get_text(strip=True, separator=' ')

    def run(self):
        self.error("")
        self.warning("")
        if self.data is None:
            return
        self.progressBarInit()
        self.thread = thread_management.Thread(self.parse_html)
        self.thread.progress.connect(self.handle_progress)
        self.thread.result.connect(self.handle_result)
        self.thread.finish.connect(self.handle_finish)
        self.thread.start()

    def handle_progress(self, progress) -> None:
        value = progress[0]
        text = progress[1]
        if value is not None:
            self.progressBarSet(value)
        if text is None:
            self.textBrowser.setText("")
        else:
            self.textBrowser.insertPlainText(text)

    def handle_result(self, result):
        data = convert.convert_json_implicite_to_data_table(result)
        self.Outputs.data.send(data)
        self.data = None

    def handle_finish(self):
        print("Generation finished")
        self.progressBarFinished()

    def post_initialized(self):
        pass

if __name__ == "__main__":
    app = QApplication(sys.argv)
    my_widget = ParseHMTL()
    my_widget.show()

    if hasattr(app, "exec"):
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())




import os
import sys
import docx
import pandas as pd

# Ajout de QLineEdit et QLabel
from AnyQt.QtWidgets import QApplication, QPushButton, QLineEdit
from Orange.widgets import widget
from Orange.widgets.utils.signals import Input, Output
from Orange.data import Domain, StringVariable, Table, DiscreteVariable

if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from Orange.widgets.orangecontrib.AAIT.utils.import_uic import uic
    from Orange.widgets.orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file
else:
    from orangecontrib.AAIT.utils.import_uic import uic
    from orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file

@apply_modification_from_python_file(filepath_original_widget=__file__)
class OWExtractTablesDocxToXlsx(widget.OWWidget):
    """
    Orange Widget qui extrait les tableaux de documents Word (.docx) et les sauvegarde
    en fichiers XLSX distincts (une table Word = un fichier XLSX).
    """
    name = "Docx to XLSX"
    description = "Extrait les tables de documents Word et les sauvegarde en XLSX, avec une option de division (split)."
    category = "AAIT - TOOLBOX"
    icon = "icons/extract_table.png"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/extract_table.png"
    gui = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/owdocxtoxlsx.ui")
    want_control_area = False
    priority = 1005

    class Inputs:
        data = Input("Files Table", Table)

    class Outputs:
        data = Output("Processed Files Table", Table)
        status_data = Output("Status Table", Table)

    def __init__(self):
        super().__init__()
        try:
            uic.loadUi(self.gui, self)
        except Exception as e:
            self.warning(f"Impossible de charger le fichier UI. {e}")

            class DummyCheckbox:
                def stateChanged(self, *args): pass

            class DummyLineEdit:
                def text(self): return ""

                def textChanged(self, *args): pass

            self.checkBox_alpha_headers = DummyCheckbox()
            self.lineEdit_trigger_text = DummyLineEdit()
            self.gui = None

        self.pushButton_run = self.findChild(QPushButton, "pushButton_run")
        if self.pushButton_run:
            self.pushButton_run.clicked.connect(self.run)

        self.lineEdit_trigger_text = self.findChild(QLineEdit, "lineEdit_trigger_text")
        self.trigger_text = ""
        if self.lineEdit_trigger_text:
            self.lineEdit_trigger_text.textChanged.connect(self._update_trigger_text)

        self.data = None
        self.autorun = True
        self.processed_statuses = []
        self.use_alpha_headers = False
        if self.gui:
            self.checkBox_alpha_headers.stateChanged.connect(self._update_alpha_headers_state)

        self.post_initialized()

    def _update_alpha_headers_state(self, state):
        self.use_alpha_headers = bool(state)

    def _update_trigger_text(self, text):
        """Met à jour la variable du texte trigger à chaque changement."""
        self.trigger_text = text.strip()

    @Inputs.data
    def set_data(self, in_data: Table | None):
        self.data = in_data
        if self.autorun:
            self.run()

    def run(self):
        if self.data is None:
            self.Outputs.data.send(None)
            self.Outputs.status_data.send(None)
            return

        self.error("")
        try:
            self.data.domain["file_path"]
        except KeyError:
            self.error("Le tableau d'entrée doit contenir une colonne 'file_path'.")
            self.Outputs.data.send(None)
            self.Outputs.status_data.send(None)
            return

        self.progressBarInit()
        self.processed_statuses = []
        self.Outputs.status_data.send(None)

        if self.trigger_text:
            self.information(f"Utilisation du trigger de split : '{self.trigger_text}'")
        else:
            self.information("Aucun trigger de split défini. Export standard.")

        result_rows = self._process_files(self.data)

        output_domain = Domain([], metas=[
            StringVariable("src_path"),
            StringVariable("output_dir_path"),
            StringVariable("status")
        ])
        result_table = Table.from_list(output_domain, result_rows)
        self.Outputs.data.send(result_table)

        self.progressBarFinished()

    def _process_files(self, in_data: Table) -> list:
        result_rows = []
        file_paths = [str(x) for x in in_data.get_column("file_path")]
        total_files = len(file_paths)

        if not file_paths:
            return []

        for i, full_path in enumerate(file_paths):
            self.progressBarSet((i + 1) / total_files * 100)

            status_short = "ko"
            details = "traitement échoué"
            output_dir_path = ""

            if not full_path.lower().endswith('.docx'):
                status_short = "skipped"
                details = "Fichier ignoré : n'est pas un fichier .docx."
                output_dir_path = "N/A"
                self.processed_statuses.append([full_path, status_short, details])
                self._send_status_table()
                result_rows.append([full_path, output_dir_path, f"{status_short}: {details}"])
                QApplication.processEvents()
                continue

            try:
                tables_found, output_dir_path = self._extraire_et_convertir(full_path)

                if tables_found > 0:
                    status_short = "ok"
                    details_suffix = "séparée(s)" if self.trigger_text else "standard(s)"
                    details = f"{tables_found} table(s) {details_suffix} extraite(s) et convertie(s) en XLSX."
                else:
                    status_short = "ko"
                    details = "Aucune table valide trouvée."

            except FileNotFoundError:
                details = "Fichier non trouvé."
            except Exception as e:
                details = f"Une erreur inattendue est survenue : {e}"

            self.processed_statuses.append([full_path, status_short, details])
            self._send_status_table()

            result_rows.append([full_path, output_dir_path, f"{status_short}: {details}"])

            QApplication.processEvents()

        return result_rows

    # --- MODIFIÉ : Logique d'extraction simplifiée ---
    def _extraire_et_convertir(self, docx_path):
        """
        Extrait les tableaux d'un document Word.
        Sauvegarde les tables (splittées ou non) dans '..._tables_data'.
        """
        dir_name, file_name = os.path.split(docx_path)
        base_name, _ = os.path.splitext(file_name)

        # --- MODIFIÉ : Un seul dossier de sortie ---
        output_dir_main = os.path.join(dir_name, base_name + '_tables_data')
        os.makedirs(output_dir_main, exist_ok=True)

        doc = docx.Document(docx_path)
        total_tables_main_found = 0

        # Récupère le trigger et le met en minuscule (ou None)
        trigger_text_lower = self.trigger_text.lower() if self.trigger_text else None

        for i, table in enumerate(doc.tables):
            raw_data = []
            for row in table.rows:
                row_data = [cell.text.strip() for cell in row.cells]
                raw_data.append(row_data)

            if not raw_data or not any(row for row in raw_data):
                continue

            table_index = i + 1

            # --- MODIFIÉ : Section 1 (sauvegarde originale) supprimée ---

            # --- Traitement de la sortie principale (split ou standard) ---
            if trigger_text_lower:
                # Cas A : Un trigger est fourni, on split
                sub_tables_data = self._split_table_data(raw_data, trigger_text_lower)

                for j, sub_table_data in enumerate(sub_tables_data):
                    table_name = f"table_{table_index}_{chr(ord('a') + j)}"
                    df_split = self._create_dataframe(sub_table_data)

                    if df_split is not None and not df_split.empty:
                        self._save_sub_table(df_split, output_dir_main, table_name)
                        total_tables_main_found += 1

            else:
                # Cas B : Pas de trigger, comportement original
                table_name = f"table_{table_index}_a"  # Garde le suffixe _a pour cohérence
                df_main = self._create_dataframe(raw_data)

                if df_main is not None and not df_main.empty:
                    self._save_sub_table(df_main, output_dir_main, table_name)
                    total_tables_main_found += 1

        return total_tables_main_found, output_dir_main

    def _split_table_data(self, raw_data: list, trigger_text: str) -> list:
        """
        Découpe une table (liste de lignes) en sous-tables logiques.
        """
        data = [row for row in raw_data if row and any(cell.strip() for cell in row)]
        if not data:
            return []

        if self.use_alpha_headers:
            # Cas A : En-têtes alphabétiques
            sub_tables = []
            if not data:
                return []

            current_table = [data[0]]
            for row in data[1:]:
                if any(trigger_text in str(cell).lower() for cell in row):
                    # self.information(f"  [+] Déclencheur '{trigger_text}' détecté (mode alpha-headers). Division.")
                    if current_table:
                        sub_tables.append(current_table)
                    current_table = [row]
                else:
                    current_table.append(row)
            if current_table:
                sub_tables.append(current_table)
            return sub_tables

        else:
            # Cas B : Première ligne = en-tête
            if len(data) <= 1:
                return [data]

            headers = data[0]
            data_rows = data[1:]
            final_tables = []
            current_data_segment = []

            if data_rows:
                current_data_segment.append(data_rows[0])  # Ligne n+1 ne splite jamais

            for row in data_rows[1:]:  # Commence à n+2
                if any(trigger_text in str(cell).lower() for cell in row):
                    # self.information(f"  [+] Déclencheur '{trigger_text}' détecté (mode 1ere-ligne-header). Division.")
                    if current_data_segment:
                        final_tables.append([headers] + current_data_segment)
                    current_data_segment = [row]
                else:
                    current_data_segment.append(row)

            if current_data_segment:
                final_tables.append([headers] + current_data_segment)

            return final_tables

    def _create_dataframe(self, data):
        """
        Crée le DataFrame à partir des lignes brutes (d'une sous-table).
        """
        # Nettoyer les lignes vides
        data = [row for row in data if row and any(cell.strip() for cell in row)]
        if not data:
            return None

        max_cols = max(len(row) for row in data)
        data = [row + [''] * (max_cols - len(row)) for row in data]

        if self.use_alpha_headers:
            # Cas A : En-têtes alphabétiques. Toutes les lignes sont des données.
            headers = [chr(ord('A') + j) for j in range(max_cols)]
            df = pd.DataFrame(data, columns=headers)
        else:
            # Cas B : Première ligne comme en-tête.
            if len(data) == 1:
                # Si le segment n'a qu'une seule ligne, on utilise des en-têtes alphabétiques.
                headers = [chr(ord('A') + j) for j in range(max_cols)]
                df = pd.DataFrame(data, columns=headers)
            else:
                # Cas standard : première ligne = en-tête, reste = données.
                headers = data[0]
                data_rows = data[1:]
                min_cols = min(len(headers), max_cols)

                if not data_rows:
                    df = pd.DataFrame(columns=headers[:min_cols])
                else:
                    data_rows_adjusted = [row[:min_cols] for row in data_rows]
                    df = pd.DataFrame(data_rows_adjusted, columns=headers[:min_cols])

            df.columns = df.columns.astype(str)

        return df

    def _save_sub_table(self, df, output_dir, table_full_name):
        """Sauvegarde le DataFrame exclusivement en XLSX."""

        output_xlsx_path = os.path.join(output_dir, f"{table_full_name}.xlsx")
        try:
            df.to_excel(output_xlsx_path, index=False, engine='openpyxl')
        except Exception as e:
            self.warning(f"Impossible de sauvegarder la table '{table_full_name}' en format XLSX : {e}")

    def _send_status_table(self):
        domain = Domain([], metas=[
            StringVariable("src_path"),
            DiscreteVariable("status", values=["ok", "ko", "skipped"]),
            StringVariable("details")
        ])
        status_table = Table.from_list(domain, self.processed_statuses)
        self.Outputs.status_data.send(status_table)

    def post_initialized(self):
        pass


if __name__ == "__main__":
    app = QApplication(sys.argv)
    my_widget = OWExtractTablesDocxToXlsx()
    my_widget.show()
    app.exec()